package me.yoqi.android.appmanager;

import android.content.Context;
import android.os.Bundle;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import me.yoqi.android.appmanager.model.AppInfo;
import me.yoqi.android.appmanager.model.AppModel;
import me.yoqi.android.appmanager.views.AppInfoAdapter;
import me.yoqi.android.utils.SPUtils;
import me.yoqi.android.utils.SimplexToast;

public class MainActivity extends AppCompatActivity implements View.OnClickListener {

    public static final String TAG = "MainActivity";
    RadioButton allApp, dsfApp;
    ListView listView;
    TextView tvLoad;
    EditText edtSearch;
    AppInfoAdapter adapter;
    List<AppInfo> mdatas = new ArrayList<>();  //当前界面app list
    List<AppInfo> appInfos = new ArrayList<>();  //系统app list
    List<AppInfo> dsfappInfos = new ArrayList<>(); //第三方app
    Context mContext;
    SimplexToast simplexToast;
    SPUtils spUtils;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        mContext = this;
        initView();
        initData();
    }

    private void initView() {
        allApp = findViewById(R.id.allApp); //所有app
        dsfApp = findViewById(R.id.dsfApp); //第三方app
        tvLoad = findViewById(R.id.tvLoad); //加载中。。。
        listView = findViewById(R.id.listView);
        edtSearch = findViewById(R.id.edt_search);
        edtSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                String searchWord = edtSearch.getText().toString().trim();
                searchApp(searchWord);
            }

            @Override
            public void afterTextChanged(Editable s) {

            }
        });
        allApp.setOnClickListener(this);
        dsfApp.setOnClickListener(this);
        simplexToast = SimplexToast.getInstance(mContext);
        spUtils = new SPUtils(mContext);
    }

    private void initData() {
        Set<String> pkgs = spUtils.getStringSet("hide_app", null);
        // 构建适配器，并且注册到listView
        adapter = new AppInfoAdapter(this, mdatas);
        listView.setAdapter(adapter);
        new Thread(new Runnable() {
            @Override
            public void run() {
                appInfos = AppModel.queryFilterAppInfo(mContext, AppModel.FILTER_SYSTEM_APP);
                dsfappInfos = AppModel.queryFilterAppInfo(mContext, AppModel.FILTER_THIRD_APP);
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        mdatas.addAll(appInfos);
                        adapter.notifyDataSetChanged();
                        tvLoad.setVisibility(View.GONE);
                        listView.setVisibility(View.VISIBLE);
                        edtSearch.setEnabled(true);
                    }
                });
            }
        }).start();
    }

    /**
     * 通过关键词搜索app
     *
     * @param keyword
     */
    private void searchApp(String keyword) {
        mdatas.clear();
        if (allApp.isChecked()) {
            for (int i = 0; i < appInfos.size(); i++) {
                if (appInfos.get(i).getAppName().toLowerCase().contains(keyword) || appInfos.get(i).getPackageName().toLowerCase().contains(keyword)) {
                    mdatas.add(appInfos.get(i));
                }
            }
        } else if (dsfApp.isChecked()) {
            for (int i = 0; i < dsfappInfos.size(); i++) {
                if (dsfappInfos.get(i).getAppName().toLowerCase().contains(keyword) || dsfappInfos.get(i).getPackageName().toLowerCase().contains(keyword)) {
                    mdatas.add(dsfappInfos.get(i));
                }
            }
        }
        adapter.notifyDataSetChanged();
    }

    @Override
    public void onClick(View v) {
        String keyword = edtSearch.getText().toString().trim();
        switch (v.getId()) {
            case R.id.allApp:
                mdatas.clear();
                if (!keyword.equals("")) {
                    for (int i = 0; i < appInfos.size(); i++) {
                        if (appInfos.get(i).getAppName().toLowerCase().contains(keyword) || appInfos.get(i).getPackageName().toLowerCase().contains(keyword)) {
                            mdatas.add(appInfos.get(i));
                        }
                    }
                } else {
                    mdatas.addAll(appInfos);
                }
                adapter.notifyDataSetChanged();
                break;
            case R.id.dsfApp:
                mdatas.clear();
                if (!keyword.equals("")) {
                    for (int i = 0; i < dsfappInfos.size(); i++) {
                        if (dsfappInfos.get(i).getAppName().toLowerCase().contains(keyword) || dsfappInfos.get(i).getPackageName().toLowerCase().contains(keyword)) {
                            mdatas.add(dsfappInfos.get(i));
                        }
                    }
                } else {
                    mdatas.addAll(dsfappInfos);
                }
                adapter.notifyDataSetChanged();
                break;
        }
    }
}